/** @module player/state */
import { call, put, takeEvery } from 'redux-saga/effects'
import { generatePlayerList } from './functions'

/**
 * state.js
 *
 * Collection of everything which has to do with state changes.
 **/

/** actionTypes define what actions are handeled by the reducer. */
export const actions = {
  fileUploadStart: file => {
    return {
      type: 'PLAYER_FILE_UPLOAD_START',
      file
    }
  },
  fileUploadSuccess: data => {
    return {
      type: 'PLAYER_FILE_UPLOAD_SUCCESS',
      data
    }
  },
  fileUploadFailure: error => {
    return {
      type: 'PLAYER_FILE_UPLOAD_FAILURE',
      alert: { type: 'warning', text: error.toString() }
    }
  },
  filterSortPlayers: () => {
    return {
      type: 'PLAYER_FILTER_SORT'
    }
  }
}
console.log('State actions', actions)

/** state definition */
export const state = {
  headers: [],
  allPlayers: [],
  filteredPlayers: [],
  filters: [
    {
      filterFunction: (player, filterValue) => {
        return player.isJunior === filterValue
      },
      filterValue: true
    }
  ],
  sorting: [
    (player1, player2) => {
      if (player1.Konkurrenz > player2.Konkurrenz) {
        return 1
      }
      if (player1.Konkurrenz === player2.Konkurrenz) {
        return 0
      }
      return -1
    },
    (player1, player2) => {
      if (player1.Vorname > player2.Vorname) {
        return 1
      }
      if (player1.Vorname === player2.Vorname) {
        return 0
      }
      return -1
    }
  ],
  fileUpload: 'idle',
  file: null
}
console.log('State state', state)

/** reducer is called by the redux dispatcher and handles all component actions */
export function reducer (state = [], action) {
  switch (action.type) {
    case 'PLAYER_FILE_UPLOAD_START':
      return { ...state, fileUpload: 'started', file: action.file }
    case 'PLAYER_FILE_UPLOAD_SUCCESS':
      return { ...state, fileUpload: 'finished', allPlayers: action.data.allPlayers, headers: action.data.headers }
    case 'PLAYER_FILE_UPLOAD_FAILURE':
      return { ...state, fileUpload: 'failure' }
    case 'PLAYER_FILTER_SORT':
      const { allPlayers, filters, sorting } = state
      const sortedPlayers = allPlayers.sort((player1, player2) => {
        return sorting.length ? sorting.reduce((acc, sortFun) => {
          return acc === 0 ? sortFun(player1, player2) : acc
        }, 0) : 0
      })
      console.log('Sorted players', sortedPlayers)
      const filteredPlayers = sortedPlayers.filter(player => {
        return filters.length ? filters.map(filter => filter.filterFunction(player, filter.filterValue)).every(value => !!value) : true
      })
      console.log('Filtered players', filteredPlayers)
      return { ...state, filteredPlayers }
    default:
      return state
  }
}

function * uploadFile (action) {
  try {
    console.log('PlayerList uploadFile', action.file)
    const allPlayers = yield call(generatePlayerList, action.file)
    console.log('PlayerList success!', actions.fileUploadSuccess(allPlayers))
    yield put(actions.fileUploadSuccess(allPlayers))
    yield put(actions.filterSortPlayers())
  } catch (error) {
    console.log('PlayerList failure!', actions.fileUploadFailure(error))
    yield put(actions.fileUploadFailure(error))
  }
}

/** sagas are asynchronous workers (JS generators) to handle the state. */
export function * saga () {
  console.log('Player saga started.')
  yield takeEvery('PLAYER_FILE_UPLOAD_START', uploadFile)
}
